@@ -104,9 +104,11 @@ struct ParseFileInfo {
     bool newinfounix_seen; // true if Info-ZIP Unix New extra field has
     // been parsed
 
-    // file sizes and header position from a ZIP64 extra field
+    // file sizes from a ZIP64 extra field
     quint64 uncompressedSize = 0;
     quint64 compressedSize = 0;
+    // position of the Local File Header itself, or from the
+    // ZIP64 extra field in the Central Directory
     quint64 localheaderoffset = 0;
 
     ParseFileInfo()
@@ -319,7 +321,7 @@ static bool seekAnyHeader(QIODevice *dev, QByteArray &header, qsizetype minSize)
     int n = dev->peek(header.data(), header.size());
 
     while (n >= minSize) {
-        if (auto i = header.indexOf("PK"); i >= 0) {
+        if (auto i = QByteArrayView(header.data(), n).indexOf("PK"); i >= 0) {
             dev->seek(dev->pos() + i);
             if ((i + minSize) < n) {
                 header.remove(0, i);
@@ -353,14 +355,16 @@ static bool seekPostDataDescriptor(QIODevice *dev, bool isZip64)
     const qsizetype descriptorSize = isZip64 ? 24 : 16;
 
     QByteArray header;
-    auto oldPos = dev->pos();
+    const qint64 oldPos = dev->pos();
 
     while (seekAnyHeader(dev, header, descriptorSize)) {
         // qCDebug(KArchiveLog) << "Possible data descriptor header at" << dev->pos() << header;
         if (header.startsWith("PK\x07\x08")) {
-            qint64 compressedSize = isZip64 ? parseUi64(header.constData() + 8) //
-                                            : parseUi32(header.constData() + 8);
-            if (oldPos + compressedSize == dev->pos()) {
+            const qint64 compressedSize = isZip64 ? parseUi64(header.constData() + 8) //
+                                                  : parseUi32(header.constData() + 8);
+            // Do not move compressedSize to the left side of the check
+            // we have on the left on purpose to prevent addition overflows
+            if ((compressedSize > 0) && (oldPos == dev->pos() - compressedSize)) {
                 dev->seek(dev->pos() + descriptorSize);
                 return true;
             }
@@ -387,6 +391,8 @@ static bool seekToNextHeaderToken(QIODevice *dev)
         // PK12 for the central header in case there is no data descriptor
         if (header.startsWith("PK\x03\x04") || header.startsWith("PK\x01\x02")) {
             return true;
+        } else {
+            dev->seek(dev->pos() + 2); // Skip found 'PK'
         }
     }
     return false;
@@ -420,6 +426,8 @@ public:
     // writeonly mode, or it points to the beginning of the central directory.
     // each call to writefile updates this value.
     quint64 m_offset;
+    // Position of the first Local File Header
+    quint64 m_startPos = 0;
 };
 
 KZip::KZip(const QString &fileName)
@@ -465,6 +473,8 @@ bool KZip::openArchive(QIODevice::OpenMode mode)
     // We set a bool for knowing if we are allowed to skip the start of the file
     bool startOfFile = true;
 
+    qint64 expectedStartPos = 0;
+
     for (;;) { // repeat until 'end of entries' signature is reached
         // qCDebug(KArchiveLog) << "loop starts";
         // qCDebug(KArchiveLog) << "dev->pos() now : " << dev->pos();
@@ -490,6 +500,9 @@ bool KZip::openArchive(QIODevice::OpenMode mode)
             // qCDebug(KArchiveLog) << "PK34 found local file header at offset" << dev->pos();
             startOfFile = false;
 
+            ParseFileInfo pfi;
+            pfi.localheaderoffset = dev->pos() - 4;
+
             // read static header stuff
             n = dev->read(buffer, 26);
             if (n < 26) {
@@ -528,7 +541,6 @@ bool KZip::openArchive(QIODevice::OpenMode mode)
                 return false;
             }
 
-            ParseFileInfo pfi;
             pfi.mtime = mtime;
             pfi.dataoffset = 30 + extralen + namelen;
 
@@ -659,7 +671,13 @@ bool KZip::openArchive(QIODevice::OpenMode mode)
                 setErrorString(tr("Invalid ZIP file, file path name length is zero"));
                 return false;
             }
-            QByteArray varData = dev->read(namelen + extralen);
+            const int varDataDesiredLength = namelen + extralen;
+            const QByteArray varData = dev->read(varDataDesiredLength);
+            if (varData.length() < varDataDesiredLength) {
+                setErrorString(tr("Invalid ZIP file, unable to read %1 + %2 bytes for filename and extra data at offset %3")
+                                   .arg(namelen, extralen, dev->pos() - varData.size()));
+                return false;
+            }
 
             ParseFileInfo extrafi;
             if (extralen) {
@@ -667,9 +685,6 @@ bool KZip::openArchive(QIODevice::OpenMode mode)
             }
 
             QByteArray bufferName(varData.constData(), namelen);
-            if (bufferName.size() < namelen) {
-                // qCWarning(KArchiveLog) << "Invalid ZIP file. Name not completely read";
-            }
 
             ParseFileInfo pfi = pfi_map.value(bufferName, ParseFileInfo());
 
@@ -703,7 +718,17 @@ bool KZip::openArchive(QIODevice::OpenMode mode)
             // qCDebug(KArchiveLog) << "localheader dataoffset: " << pfi.dataoffset;
 
             // offset, where the real data for uncompression starts
-            qint64 dataoffset = localheaderoffset + pfi.dataoffset; // comment only in central header
+            qint64 dataoffset = localheaderoffset + pfi.dataoffset;
+            if (pfi.localheaderoffset != expectedStartPos + localheaderoffset) {
+                if (pfi.localheaderoffset == d->m_startPos + localheaderoffset) {
+                    qCDebug(KArchiveLog) << "warning:" << d->m_startPos << "extra bytes at beginning of zipfile";
+                    expectedStartPos = d->m_startPos;
+                } else {
+                    setErrorString(tr("Invalid ZIP file, inconsistent Local Header Offset in Central Directory at %1").arg(offset));
+                    return false;
+                }
+                dataoffset = localheaderoffset + pfi.dataoffset + d->m_startPos;
+            }
 
             // qCDebug(KArchiveLog) << "csize: " << csize;
 
@@ -806,12 +831,17 @@ bool KZip::openArchive(QIODevice::OpenMode mode)
                  */
                 if (header.startsWith("PK\x03\x04")) {
                     foundSignature = true;
+                    d->m_startPos = dev->pos();
                     break;
                 }
+                if (dev->pos() > 4 * 1024 * 1024) {
+                    break;
+                }
+                dev->seek(dev->pos() + 2); // Skip found 'PK'
             }
 
             if (!foundSignature) {
-                setErrorString(tr("Invalid ZIP file. Unexpected end of file."));
+                setErrorString(tr("Not a ZIP file, no Local File Header found."));
                 return false;
             }
         } else {
@@ -1453,14 +1483,14 @@ QIODevice *KZipFileEntry::createDevice() const
 {
     // qCDebug(KArchiveLog) << "creating iodevice limited to pos=" << position() << ", csize=" << compressedSize();
     // Limit the reading to the appropriate part of the underlying device (e.g. file)
-    KLimitedIODevice *limitedDev = new KLimitedIODevice(archive()->device(), position(), compressedSize());
+    std::unique_ptr limitedDev = std::make_unique<KLimitedIODevice>(archive()->device(), position(), compressedSize());
     if (encoding() == 0 || compressedSize() == 0) { // no compression (or even no data)
-        return limitedDev;
+        return limitedDev.release();
     }
 
     if (encoding() == 8) {
         // On top of that, create a device that uncompresses the zlib data
-        KCompressionDevice *filterDev = new KCompressionDevice(limitedDev, true, KCompressionDevice::GZip);
+        KCompressionDevice *filterDev = new KCompressionDevice(std::move(limitedDev), KCompressionDevice::GZip, size());
 
         if (!filterDev) {
             return nullptr; // ouch
@@ -1474,6 +1504,5 @@ QIODevice *KZipFileEntry::createDevice() const
 
     qCCritical(KArchiveLog) << "This zip file contains files compressed with method" << encoding() << ", this method is currently not supported by KZip,"
                             << "please use a command-line tool to handle this file.";
-    delete limitedDev;
     return nullptr;
 }
